@@ -1,7 +1,7 @@
-#!/usr/bin/python
+#!/usr/bin/env python3
 # coding: utf-8
 from xdg.DesktopEntry import DesktopEntry
-from xdg.Exceptions import ValidationError
+from xdg.Exceptions import ValidationError, ParsingError, NoKeyError
 from xdg.util import u
 
 import resources
@@ -37,7 +37,7 @@ class DesktopEntryTest(unittest.TestCase):
             contents = f.read()
         
         assert "[Hallo]" in contents, contents
-        assert re.search("key\s*=\s*value", contents), contents
+        assert re.search(r"key\s*=\s*value", contents), contents
         
         # This is missing the Name key, and has an unknown Hallo group, so it
         # shouldn't validate.
@@ -77,3 +77,46 @@ class DesktopEntryTest(unittest.TestCase):
         
         entry = DesktopEntry(self.test_file)
         self.assertEqual(entry.getName(), u('Abc€þ'))
+    
+    def test_invalid(self):
+        test_file = os.path.join(self.tmpdir, "spout.desktop")
+        with io.open(test_file, "w", encoding='utf-8') as f:
+            f.write(resources.spout_desktop)
+        
+        self.assertRaises(ParsingError, DesktopEntry, test_file)
+
+    def test_invalid_unicode(self):
+        test_file = os.path.join(self.tmpdir, "gnome-alsamixer.desktop")
+        with io.open(test_file, "w", encoding='latin-1') as f:
+            f.write(resources.gnome_alsamixer_desktop)
+        
+        # Just check this doesn't throw a UnicodeError.
+        DesktopEntry(test_file)
+
+class TestTryExec(unittest.TestCase):
+    def setUp(self):
+        self.tmpdir = tempfile.mkdtemp()
+        self.test_file = os.path.join(self.tmpdir, "foo.desktop")
+
+    def test_present(self):
+        with io.open(self.test_file, "w", encoding='utf-8') as f:
+            f.write(resources.python_desktop)
+        
+        entry = DesktopEntry(self.test_file)
+        res = entry.findTryExec()
+        assert res, repr(res)
+
+    def test_absent(self):
+        with io.open(self.test_file, "w", encoding='utf-8') as f:
+            f.write(resources.unicode_desktop)
+        
+        entry = DesktopEntry(self.test_file)
+        res = entry.findTryExec()
+        assert res is None, repr(res)
+        
+    def test_no_TryExec(self):
+        with io.open(self.test_file, "w", encoding='utf-8') as f:
+            f.write(resources.gedit_desktop)
+        
+        entry = DesktopEntry(self.test_file)
+        self.assertRaises(NoKeyError, entry.findTryExec)
